/* Copyright (C) Peter Rank Software - All Rights Reserved
 * Written by Peter Rank <peter@softmanufaktur.de>, 2016
 */
import React from 'react';
import SwipeCanvas from '../swipecanvas/swipecanvas.js';
import Helper from '../helper/helper.js'
import TableEvent from '../table/tableevent.js';

/**
 *
 **/
class Table extends SwipeCanvas {
    constructor(props) {
        super(props);

        this.rowOffset = 0;
        this.workRowOffset = 0;

        this.colOffset = 0;
        this.workColOffset = 0;

        this.totalTableWidth = undefined;

        this.columnWidth = [];

        this.selectedRow = this.props.selectedRow >= 0 ? this.props.selectedRow : -1;

        this.props.model.addDataChangeCallback(function () {
            this.totalTableWidth = undefined;
            this.totalTableHeight = undefined;
            //SELF.paintStartX = undefined;
            //SELF.paintStartY = undefined;
            this.columnWidth = [];
            this.paint();
        }.bind(this));

        this.totalTableHeight = this.recomputeTotalTableHeight(this.props.model);
    }

    componentWillReceiveProps(nextProps) {
        this.props.model.addDataChangeCallback(function () {
            this.totalTableWidth = undefined;
            this.totalTableHeight = this.recomputeTotalTableHeight(nextProps.model);
            this.columnWidth = [];
            this._updateCanvas();
        }.bind(this));

        this.selectedRow = nextProps.selectedRow >= 0 ? nextProps.selectedRow : -1;
        this.totalTableHeight = this.recomputeTotalTableHeight(nextProps.model);
        this._updateCanvas();
    }

    componentDidMount() {
        super.componentDidMount();
    }

    //Das Default-Verhalten (Zoom) wird überschrieben, damit mit dem Wheelrad in der Tabelle gescrolled werden kann
    _wheel(evt) {
        this.offsetY += -evt.deltaY;

        this.offsetChanged();
        this.offsetY = 0;
        this.offsetResetted();

        this._updateCanvas();

        //Wenn gescrolled wird, dann darf nicht der ganze Bildschirm mitgescrolled werden
        let checkHeight = Math.max(this.props.height - this.props.model.getHeaderHeight(), this.totalTableHeight);
        if (!((evt.deltaY < 0 && this.workRowOffset === 0) || (evt.deltaY > 0 && this.workRowOffset <= this.props.height - checkHeight - this.props.model.getHeaderHeight()))) {
            evt.stopImmediatePropagation();
            evt.preventDefault();
            evt.returnValue = false;
            return false;
        }
    }

    _tap(evt) {
        super._tap(evt);
        let tableEvt = this._createTableEvent(evt);
        //this.selectedRow = tableEvt.getRow();
        if (this.props.onTap) {
            this.props.onTap(tableEvt);
        }

    }

    _createTableEvent(evt) {
        let mousePos = Helper.getCursorPosition(this.refs.canvas, evt);
        let x = mousePos[0];
        let y = mousePos[1];

        let retVal = new TableEvent();

        //Geklickte Spalte suchen
        let xStart = this.workColOffset;
        for (let col = 0; col < this.props.model.getColumnCount(); col++) {
            let width = this._getColumnWidth(col);
            if (x >= xStart && x <= xStart + width) {
                retVal.setColumn(col);
                break;
            }

            xStart += width;
        }

        //Geklickte Zeile suchen
        let yStart = this.workRowOffset + this.props.model.getHeaderHeight();
        for (let row = 0; row < this.props.model.getRowCount(); row++) {
            let height = this.props.model.getRowHeight(row);
            if (y >= yStart && y <= yStart + height) {
                retVal.setRow(row);
                break;
            }

            yStart += height;
        }

        return retVal;
    }

    /*getSelectedRow() {
        return this.selectedRow;
    }*/

    offsetResetted() {
        //this._alignWorkResOffset();
        this.rowOffset = this.workRowOffset;
        this.colOffset = this.workColOffset;
        super.offsetResetted();
    }

    offsetChanged() {
        this.workRowOffset = this.rowOffset + this.offsetY;
        this.workColOffset = this.colOffset + this.offsetX;

        if (this.workRowOffset > 0) {
            this.workRowOffset = 0;
        }

        let checkHeight = Math.max(this.props.height - this.props.model.getHeaderHeight(), this.recomputeTotalTableHeight(this.props.model));
        if (this.workRowOffset < this.props.height - checkHeight - this.props.model.getHeaderHeight()) {
            this.workRowOffset = this.props.height - checkHeight - this.props.model.getHeaderHeight()
        }
        if (this.workColOffset > 0) {
            this.workColOffset = 0;
        }
        if (this.workColOffset < this.props.width - this.getTotalTableWidth()) {
            this.workColOffset = this.props.width - Math.max(this.getTotalTableWidth(), this.props.width);
        }
        //this._alignWorkResOffset();
        super.offsetChanged();
        //this._fireOffsetChanged();
    }

    getTotalTableWidth() {
        if (this.totalTableWidth === undefined) {
            this.totalTableWidth = 0;
            for (let col = 0; col < this.props.model.getColumnCount(); col++) {
                this.totalTableWidth += this._getColumnWidth(col);
            }
        }
        return this.totalTableWidth;
    }

    recomputeTotalTableHeight(model) {

        this.totalTableHeight = 0;
        for (let row = 0; row < model.getRowCount(); row++) {
            this.totalTableHeight += model.getRowHeight(row);
        }

        return this.totalTableHeight;
    }

    getPaintStartX() {
        //TODO: das muss nicht immer komplett berechnet werden
        //if(this.paintStartX === undefined) {
        let xStart = 0;
        let startCol = 0;
        for (let col = 0; col < this.props.model.getColumnCount(); col++) {
            let width = this._getColumnWidth(col);
            xStart += width;
            if (xStart > -this.workColOffset) {
                xStart -= width;
                startCol = col;
                break;
            }
        }
        this.paintStartX = [];
        this.paintStartX.xStart = xStart;
        this.paintStartX.startCol = startCol;
        //}

        return this.paintStartX;
    }

    getTotalFixedColumnWidth() {
        let width = 0;
        for (let col = 0; col < this.props.model.getColumnCount() && col < this.props.model.getFixedColumns() - 1; col++) {
            width += this._getColumnWidth(col);
        }
        return width;
    }

    getPaintStartY() {
        //TODO: das muss nicht immer komplett berechnet werden
        //if(this.paintStartY === undefined) {
        let yStart = this.props.model.getHeaderHeight();
        let startRow = 0;
        for (let row = 0; row < this.props.model.getRowCount(); row++) {
            let height = this.props.model.getRowHeight(row);
            yStart += height;
            if (yStart > -this.workRowOffset) {
                yStart -= height;
                startRow = row;
                break;
            }
        }
        this.paintStartY = [];
        this.paintStartY.yStart = yStart;
        this.paintStartY.startRow = startRow;
        //}


        return this.paintStartY;
    }

    //Liefert die Spaltenbreite, falls diese >=0 angegeben wurde, oder berechnet diese anhand der Daten, falls -1 angegeben wurde
    _getColumnWidth(col) {
        let width = this.props.model.getColumnWidth(col);
        if (this.ctx && width < 0) {
            width = this.columnWidth[col];

            if (width === undefined) {
                //Bestimmen der Spaltenbreite (Längste Zeichenkette)
                for (let row = 0; row < this.props.model.getRowCount(); row++) {
                    let val = this.props.model.getValueAt(row, col);
                    let txtWidth;
                    if (this.props.cellSpacer) {
                        txtWidth = this.props.cellSpacer(this.ctx, val, row, col);
                    } else {
                        txtWidth = this.ctx.measureText(val).width;
                    }

                    if (width === undefined || txtWidth > width) {
                        width = txtWidth;
                        this.columnWidth[col] = width;
                    }
                }

                //Immer noch undefined? -> Weil Breite berechnet werden soll, aber noch keine Zeilen da sind?
                if (width === undefined) {
                    width = Math.round(this.props.width / this.props.model.getColumnCount());
                }
            }
        }
        return width;
    }

    paintScrollBar(ctx) {
        if (this.props.model !== undefined) {

            ctx.fillStyle = "rgba(200, 200, 200, 0.5)";

            let totalScrollbarHeight = this.props.height - this.props.model.getHeaderHeight();
            let factor = totalScrollbarHeight / this.totalTableHeight;
            let barSize = totalScrollbarHeight * factor;
            if (barSize < totalScrollbarHeight) {
                ctx.fillStyle = "rgba(200, 200, 200, 0.5)";
                ctx.fillRect(this.props.width - 10, this.props.model.getHeaderHeight(), 10, this.props.height - this.props.model.getHeaderHeight());

                ctx.fillStyle = "rgba(50, 50, 50, 0.7)";
                ctx.fillRect(this.props.width - 10, -this.workRowOffset * factor + this.props.model.getHeaderHeight(), 10, barSize);
            }

        }
    }

    paint() {
        this.ctx.save();

        this.ctx.clearRect(0, 0, this.props.width, this.props.height);

        this.ctx.fillStyle = this.props.headercolor || "#AAAAAA";
        this.ctx.strokeStyle = this.props.gridcolor || "#AAAAAA";
        this.ctx.beginPath();

        let headerHeight = this.props.model.getHeaderHeight();
        this.ctx.fillRect(0, 0, this.props.width, headerHeight);


        this.ctx.fillStyle = "#000000";

        //Bestimme die erste Spalte, mit der das Zeichnen beginnen soll
        let paintStartX = this.getPaintStartX();

        //Bestimme die erste Zeile, mit der das Zeichnen beginnen soll
        let paintStartY = this.getPaintStartY();


        //Fixe Spalten mit Header zeichnen
        let fixedColStart = 0;
        for (let col = 0; col < this.props.model.getColumnCount() && col < this.props.model.getFixedColumns(); col++) {
            let width = this._getColumnWidth(col);

            this.ctx.moveTo(fixedColStart, 0);
            this.ctx.lineTo(fixedColStart, this.props.height);

            this.ctx.save();
            this.ctx.rect(fixedColStart, 0, width, headerHeight);
            this.ctx.clip();
            if (this.props.headerRenderer) {
                this.props.headerRenderer(this.ctx, this.props.model.getColumnName(col), col, fixedColStart, width, headerHeight);
            } else {
                this.ctx.fillText(this.props.model.getColumnName(col), fixedColStart + 2, 20);
            }
            this.ctx.restore();

            fixedColStart += width;
        }
        this.ctx.stroke();

        this.ctx.save();

        this.ctx.beginPath();
        this.ctx.rect(0, headerHeight, fixedColStart, this.props.height - headerHeight);
        this.ctx.clip();


        //Zeileninhalte der fixen Spalten zeichnen
        fixedColStart = 0;
        for (let col = 0; col < this.props.model.getColumnCount() && col < this.props.model.getFixedColumns(); col++) {
            let width = this._getColumnWidth(col);

            y = paintStartY.yStart;
            for (let row = paintStartY.startRow; row < this.props.model.getRowCount(); row++) {
                let height = this.props.model.getRowHeight(row);
                let rowStart = this.workRowOffset + y;
                if (rowStart >= this.props.height) {
                    break;
                }

                this.ctx.save();
                this.ctx.beginPath();
                this.ctx.rect(fixedColStart, rowStart, width, height);
                this.ctx.clip();

                if (this.props.cellRenderer) {
                    this.props.cellRenderer(this.ctx, this.props.model.getValueAt(row, col), row, col, fixedColStart, rowStart, width, height, this.selectedRow === row);
                } else {
                    this.ctx.fillText(this.props.model.getValueAt(row, col), fixedColStart + 2, rowStart + 20);
                    this.ctx.stroke();
                }

                this.ctx.restore();
                y += height;
            }

            fixedColStart += width;
        }

        this.ctx.restore();

        //Zeilen zeichnen
        let y = paintStartY.yStart;
        for (let row = paintStartY.startRow; row < this.props.model.getRowCount(); row++) {
            let height = this.props.model.getRowHeight(row);
            let rowStart = this.workRowOffset + y;
            if (rowStart >= this.props.height) {
                break;
            }
            if (rowStart > headerHeight) {
                this.ctx.moveTo(0, rowStart);
                this.ctx.lineTo(this.props.width, rowStart);
            }
            y += height;
        }
        //nach der letzten Zeile
        let rowStart = this.workRowOffset + y;
        if (rowStart <= this.props.height && rowStart > headerHeight) {
            this.ctx.moveTo(0, rowStart);
            this.ctx.lineTo(this.props.width, rowStart);
        }

        this.ctx.stroke();

        this.ctx.beginPath();
        this.ctx.rect(fixedColStart, 0, this.props.width, this.props.height);
        this.ctx.clip();

        this.ctx.beginPath();
        //Header und Spalten zeichnen
        let x = paintStartX.xStart;
        for (let col = paintStartX.startCol; col < this.props.model.getColumnCount(); col++) {
            let width = this._getColumnWidth(col);
            let colStart = this.workColOffset + x;
            if (colStart >= this.props.width) {
                break;
            }

            if (col >= this.props.model.getFixedColumns()) {
                this.ctx.moveTo(colStart, 0);
                this.ctx.lineTo(colStart, this.props.height);

                this.ctx.save();
                this.ctx.rect(colStart, 0, width, headerHeight);
                this.ctx.clip();
                if (this.props.headerRenderer) {
                    this.props.headerRenderer(this.ctx, this.props.model.getColumnName(col), col, colStart, width, headerHeight);
                } else {
                    this.ctx.fillText(this.props.model.getColumnName(col), colStart + 2, 20);
                }
                this.ctx.restore();
            }

            x += width;
        }

        this.ctx.stroke();

        this.ctx.beginPath();
        this.ctx.rect(fixedColStart, headerHeight, this.props.width, this.props.height - headerHeight);
        this.ctx.clip();

        //Zeileninhalte zeichnen
        x = paintStartX.xStart;
        for (let col = paintStartX.startCol; col < this.props.model.getColumnCount(); col++) {
            let width = this._getColumnWidth(col);
            let colStart = this.workColOffset + x;
            if (colStart >= this.props.width) {
                break;
            }

            y = paintStartY.yStart;
            for (let row = paintStartY.startRow; row < this.props.model.getRowCount(); row++) {
                let height = this.props.model.getRowHeight(row);
                let rowStart = this.workRowOffset + y;
                if (rowStart >= this.props.height) {
                    break;
                }

                this.ctx.save();
                this.ctx.beginPath();
                //this.ctx.translate(-fixedColStart, -rowStart);
                this.ctx.rect(colStart, rowStart, width, height);
                this.ctx.clip();

                if (this.props.cellRenderer) {
                    this.props.cellRenderer(this.ctx, this.props.model.getValueAt(row, col), row, col, colStart, rowStart, width, height, this.selectedRow === row);
                } else {
                    this.ctx.fillText(this.props.model.getValueAt(row, col), colStart + 2, rowStart + 20);
                    this.ctx.stroke();
                }

                this.ctx.restore();

                y += height;
            }

            x += width;
        }

        this.ctx.restore();

        this.paintScrollBar(this.ctx);
    }

}

export default Table
